from .PageObjects import PageObjects  # noqa: F401
from .baseobjects import BasePage  # noqa: F401
from .PageObjectLibrary import _PageObjectLibrary  # noqa: F401
from .PageObjects import pageobject  # noqa: F401
from .BasePageObjects import ListingPage, HomePage, DetailPage, NewModal  # noqa: F401



"""
    This hybrid library/listener can be used to verify
    messages that have been logged.

    This works by listening for log messages via the listener
    interface, and saving them in a cache. Keywords are provided
    for searching the cache, and for resetting the cache.
"""
from robot.libraries.BuiltIn import BuiltIn
import re


class TestListener(object):
    ROBOT_LIBRARY_SCOPE = "TEST SUITE"
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self
        self.log_messages = []
        self.capture_log = True

    def _log_message(self, message):
        """Called whenever a message is added to the log"""
        if self.capture_log:
            self.log_messages.append(message)

    def reset_robot_log_cache(self):
        self.log_messages = []

    def assert_robot_log(self, message_pattern, log_level=None):
        """Assert that a message matching the pattern was emitted"""
        BuiltIn().log_to_console("\n")
        for message in self.log_messages:
            # note: message is a dictionary with the following keys:
            # 'timestamp', 'message', 'level', 'html'
            if re.search(message_pattern, message["message"], re.MULTILINE):
                if log_level is None or message["level"] == log_level:
                    return True
        raise Exception(
            "Could not find a robot log message matching the pattern '{}'".format(
                message_pattern
            )
        )



import io
import logging
import sarge
import sys

logger = logging.getLogger(__name__)


def sfdx(
    command,
    username=None,
    log_note=None,
    access_token=None,
    args=None,
    env=None,
    capture_output=True,
    check_return=False,
):
    """Call an sfdx command and capture its output.

    Be sure to quote user input that is part of the command using `sarge.shell_format`.

    Returns a `sarge` Command instance with returncode, stdout, stderr
    """
    command = "sfdx {}".format(command)
    if args is not None:
        for arg in args:
            command += " " + sarge.shell_quote(arg)
    if username:
        command += sarge.shell_format(" -u {0}", username)
    if log_note:
        logger.info("{} with command: {}".format(log_note, command))
    # Avoid logging access token
    if access_token:
        command += sarge.shell_format(" -u {0}", access_token)
    p = sarge.Command(
        command,
        stdout=sarge.Capture(buffer_size=-1) if capture_output else None,
        stderr=sarge.Capture(buffer_size=-1) if capture_output else None,
        shell=True,
        env=env,
    )
    p.run()
    if capture_output:
        p.stdout_text = io.TextIOWrapper(p.stdout, encoding=sys.stdout.encoding)
        p.stderr_text = io.TextIOWrapper(p.stderr, encoding=sys.stdout.encoding)
    if check_return and p.returncode:
        raise Exception(f"Command exited with return code {p.returncode}")
    return p
